@@ -157,7 +157,7 @@ pub(super) fn parse_trade(market_type: MarketType, msg: &str) -> Result<Vec<Trad
         quantity_contract: Some(size),
         side,
         trade_id: timestamp.to_string(),
-        raw: serde_json::to_value(&raw_trade).unwrap(),
+        json: msg.to_string(),
     };
 
     Ok(vec![trade])
@@ -204,7 +204,7 @@ pub(crate) fn parse_l2(market_type: MarketType, msg: &str) -> Result<Vec<OrderBo
             .map(|x| parse_order(x))
             .collect::<Vec<Order>>(),
         snapshot: false,
-        raw: serde_json::from_str(msg)?,
+        json: msg.to_string(),
     };
 
     Ok(vec![orderbook])
